"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var object_deep_recreate_1 = require("object-deep-recreate");
var storeContext_1 = require("./storeContext");
var helpers_1 = require("./helpers");
var getters_1 = require("./getters");
var externalStore_1 = require("./externalStore");
var withStore = function (Component, store, options) {
    if (options === void 0) { options = {}; }
    return function (props) {
        var initialValues = react_1.useRef(getters_1.getGettersInitialValues(store));
        var globalStoreRef = react_1.useRef(externalStore_1.createStore(store.state));
        var globalGettersRef = react_1.useRef(externalStore_1.createStore(initialValues.current));
        var prevGettersRef = react_1.useRef(JSON.parse(JSON.stringify(initialValues.current)));
        var handleGettersValuesSet = react_1.useCallback(function (newValues) {
            getters_1.calcAndSetGettersValues(store, newValues, globalGettersRef.current, prevGettersRef);
        }, [options.localStorageName]);
        react_1.useEffect(function () {
            var stateInitialValues = helpers_1.getStoreStateWithModules(store);
            if (options.localStorageName && stateInitialValues) {
                helpers_1.handleStateFillWithLocalValues(stateInitialValues, options.localStorageName);
            }
            globalStoreRef.current.setState(function () { return stateInitialValues; });
        }, []);
        var mutations = react_1.useMemo(function () {
            return getMutations(store, globalStoreRef.current, handleGettersValuesSet, options.localStorageName);
        }, [handleGettersValuesSet]);
        var actions = react_1.useMemo(function () {
            var actionsFns = helpers_1.getStoreKeyModuleValues(store, 'actions');
            return actionsFns !== null && actionsFns !== void 0 ? actionsFns : {};
        }, []);
        var MemoizedComponent = react_1.useMemo(function () { return react_1.memo(Component); }, []);
        return (react_1.default.createElement(storeContext_1.MutationsProvider, { value: mutations },
            react_1.default.createElement(storeContext_1.ActionsProvider, { value: actions },
                react_1.default.createElement(storeContext_1.GlobalStoreProvider, { value: globalStoreRef.current },
                    react_1.default.createElement(storeContext_1.GlobalGettersProvider, { value: globalGettersRef },
                        react_1.default.createElement(MemoizedComponent, __assign({}, props)))))));
    };
};
var getMutations = function (storeConfig, globalStore, handleGettersValuesSet, storageName) {
    var mutations = helpers_1.getStoreKeyModuleValues(storeConfig, 'mutations');
    var mutationNames = Object.keys(mutations);
    if (!mutationNames.length)
        return {};
    var values = {};
    mutationNames.forEach(function (mutationName) {
        var originalFn = mutations[mutationName];
        values[mutationName] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (!globalStore) {
                throw new Error('No store found');
            }
            var setter = function (state) {
                var prevStateCloned = JSON.parse(JSON.stringify(state));
                var moduleNames = mutationName.split('/');
                // alter the state with the logic given in the store config
                if (moduleNames.length === 1) {
                    originalFn.apply(void 0, __spreadArray([state], args));
                }
                else {
                    var moduleName = helpers_1.getStoreModuleName(mutationName);
                    var moduleState = helpers_1.getStoreModule(state, moduleName);
                    originalFn.apply(void 0, __spreadArray([moduleState], args));
                }
                var newValues = object_deep_recreate_1.deepRecreate(state, prevStateCloned);
                handleGettersValuesSet(newValues);
                if (storageName) {
                    localStorage.setItem(storageName, JSON.stringify(newValues));
                }
                return newValues;
            };
            globalStore.setState(setter);
        };
    });
    return values;
};
exports.default = withStore;
